/*
 *
 *  Copyright (C) 2005 Mike Isely <isely@pobox.com>
 *  Copyright (C) 2004 Aurelien Alleaume <slts@free.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "pvrusb2-i2c.h"
#include "pvrusb2-saa7115.h"
#include "pvrusb2-hdw-internal.h"
#include "pvrusb2-debug.h"
#include "saa7115.h"
#include "ivtv.h"
#include <linux/videodev.h>
#include <linux/video_decoder.h>

#define pvr2_decoder_trace(...) pvr2_trace(PVR2_TRACE_DECODER,__VA_ARGS__)

int pvr2_decoder_set_input(struct pvr2_hdw *hdw)
{
    int v = 0;
    pvr2_decoder_trace("pvr2_decoder_set_input(%d)",
		       hdw->controls[PVR2_CID_INPUT].value);
    switch(hdw->controls[PVR2_CID_INPUT].value){
    case PVR2_CVAL_INPUT_TV:
	v = 4;
	break;
    case PVR2_CVAL_INPUT_COMPOSITE:
	v = 5;
	break;
    case PVR2_CVAL_INPUT_SVIDEO:
	v = 8;
	break;
    case PVR2_CVAL_INPUT_RADIO:
	// ????? No idea yet what to do here
    default:
	return -EINVAL;
    }
    return pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_INPUT,&v);
}

int pvr2_decoder_set_bcsh(struct pvr2_hdw *hdw)
{
    struct saa7115_state st;
    int ret;
    pvr2_decoder_trace("pvr2_decoder_set_bcsh b=%d c=%d s=%d h=%d",
		       hdw->controls[PVR2_CID_BRIGHTNESS].value,
		       hdw->controls[PVR2_CID_CONTRAST].value,
		       hdw->controls[PVR2_CID_SATURATION].value,
		       hdw->controls[PVR2_CID_HUE].value);
    st.bright = hdw->controls[PVR2_CID_BRIGHTNESS].value;
    st.contrast = hdw->controls[PVR2_CID_CONTRAST].value;
    st.sat = hdw->controls[PVR2_CID_SATURATION].value;
    st.hue = hdw->controls[PVR2_CID_HUE].value;
    ret = pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_PICTURE,&st);
    return ret;
}

int pvr2_decoder_is_tuned(struct pvr2_hdw *hdw)
{
    int status;
    int ret;
    ret = pvr2_i2c_saa7115_cmd(hdw,DECODER_GET_STATUS,&status);
    if (ret < 0) return 0;
    if (status & DECODER_STATUS_GOOD) return 1;
    return 0;
}

int pvr2_decoder_set_size(struct pvr2_hdw *hdw)
{
    struct video_window wind;
    wind.width = hdw->controls[PVR2_CID_HRES].value;
    wind.height = hdw->controls[PVR2_CID_VRES].value;
    pvr2_decoder_trace("pvr2_decoder_set_size(%dx%d)",wind.width,wind.height);
    return pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_SIZE,&wind);
}

int pvr2_decoder_set_audio(struct pvr2_hdw *hdw)
{
    int val;
    pvr2_decoder_trace("pvr2_decoder_set_audio %d",
		       hdw->controls[PVR2_CID_SRATE].value);
    switch (hdw->controls[PVR2_CID_SRATE].value) {
    default:
    case PVR2_CVAL_SRATE_48:
	val = DECODER_AUDIO_48_KHZ;
	break;
    case PVR2_CVAL_SRATE_44_1:
	val = DECODER_AUDIO_441_KHZ;
	break;
    }
    return pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_AUDIO,&val);
}

int pvr2_decoder_set_norm(struct pvr2_hdw *hdw)
{
    int val;
    pvr2_decoder_trace("pvr2_decoder_set_norm %d",
		       hdw->controls[PVR2_CID_VIDEOSTANDARD].value);
    switch (hdw->controls[PVR2_CID_VIDEOSTANDARD].value) {
    default:
    case PVR2_CVAL_VIDEOSTANDARD_NTSC_M:
	val = VIDEO_MODE_NTSC;
	break;
    case PVR2_CVAL_VIDEOSTANDARD_SECAM_L:
	val = VIDEO_MODE_SECAM;
	break;
    case PVR2_CVAL_VIDEOSTANDARD_PAL_I:
    case PVR2_CVAL_VIDEOSTANDARD_PAL_DK:
    case PVR2_CVAL_VIDEOSTANDARD_PAL_BG:
	val = VIDEO_MODE_PAL;
	break;
    }
    return pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_NORM,&val);
}

#ifdef notdef
    // This causes a null pointer crash inside of saa7115.ko
static int pvr2_decoder_set_vbi(struct pvr2_hdw *hdw)
{
    int val = 0;
    pvr2_decoder_trace("pvr2_decoder_set_vbi %d",pvr->config == _vbi);
    if (hdw->config == pvr2_config_vbi) {
	if (hdw->video.standard == PVR_STD_NTSC) {
	    val = IVTV_SLICED_CAPTION_525;
	} else {
	    val = IVTV_SLICED_CAPTION_625;
	}
    }
    return pvr2_i2c_saa7115_cmd(hdw,DECODER_SET_VBI,(void *)val);
}
#endif

